@@ -3,7 +3,7 @@ use strict;
 use warnings;
 use 5.008001;
 use parent qw/Exporter/;
-our $VERSION = '0.20';
+our $VERSION = '0.21';
 use HTML::TreeBuilder::XPath;
 use LWP::UserAgent;
 use HTML::Selector::XPath 0.06 qw/selector_to_xpath/;
@@ -240,7 +240,9 @@ sub filter {
         return $self;
 
     } else {
-        return $self->find($_[0])
+        my $xpath = selector_to_xpath($_[0]);
+        my @new = grep { $_->matches($xpath) } @{$self->{trees}};        
+        return (ref $self || $self)->new_from_element(\@new, $self);
     }
 }
 
@@ -425,6 +427,27 @@ sub clone {
     return (ref $self || $self)->new_from_element(\@clones);
 }
 
+sub add {
+    my ($self, @stuff) = @_;
+    my @nodes;
+    
+    # add(selector, context)
+    if (@stuff == 2 && !ref $stuff[0] && $stuff[1]->isa('HTML::Element')) {
+        push @nodes, $stuff[1]->findnodes(selector_to_xpath($stuff[0]), root => './');        
+    }
+    else {
+        # handle any combination of html string, element object and web::query object
+        push @nodes, map { 
+            $self->{need_delete} = 1 if $_->{need_delete};
+            delete $_->{need_delete};
+            @{$_->{trees}}; 
+        } map { (ref $self || $self)->new($_) } @stuff;                
+    }
+    
+    push @{$self->{trees}}, @nodes;
+    $self;
+}
+
 sub DESTROY {
     if ($_[0]->{need_delete}) {
         $_->delete for @{$_[0]->{trees}}; # avoid memory leaks
@@ -521,6 +544,32 @@ Create new instance of Web::Query from file name.
 
 =head2 TRAVERSING
 
+=head3 add
+
+Add elements to the set of matched elements.
+
+=over 4
+
+=item add($html)
+
+An HTML fragment to add to the set of matched elements.
+
+=item add(@elements)
+
+One or more @elements to add to the set of matched elements.
+
+=item add($wq)
+
+An existing Web::Query object to add to the set of matched elements.
+
+=item add($selector, $context)
+
+$selector is a string representing a selector expression to find additional elements to add to the set of matched elements.
+
+$context is the point in the document at which the selector should begin matching
+
+=back
+
 =head3 contents
 
 Get the immediate children of each element in the set of matched elements, including text and comment nodes.
